/***************************************************************************
                          kpictorial.cpp  -  description
                             -------------------
    begin                : Thu Apr 18 21:10:17 MEST 2002
    copyright            : (C) 2002 by Roland Lezuo
    email                : roland.lezuo@chello.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kpictorial.h"


/*
   	This constructs the main window
*/
Kpictorial::Kpictorial(QWidget *parent, const char *name) : KMainWindow(parent, name)
{
	KSelectAction *diff_menu;
	QStringList diff_menu_entries;

	//we need some random numbers
	srand(KApplication::random());
	view = NULL;
	//TODO:
	config = kapp->config();



	KStdAction::open(this, SLOT( slotNewDoc() ), actionCollection(), "open");
	KStdAction::quit(this, SLOT( slotQuit() ), actionCollection(), "quit");
	KStdAction::saveAs(this, SLOT( slotSaveAs() ),actionCollection(), "saveas");
	KStdAction::showToolbar(this, SLOT( slotToggleToolbar() ), actionCollection(), "toggletoolbar");
	KStdAction::showStatusbar(this, SLOT( slotToggleStatusbar() ), actionCollection(), "togglestatusbar");
	new KAction("&Random", "kpictorial_random.png" , "r", this, SLOT( slotNewDocRandom() ), actionCollection(), "random");

	//Difficulty Menu
	diff_menu = new KSelectAction("Difficulty", NULL, "", actionCollection(), "difficulty");
	diff_menu_entries.append("Easy");
	diff_menu_entries.append("Medium");
	diff_menu_entries.append("Hard");
	diff_menu_entries.append("Harder");
	diff_menu_entries.append("Random");
	diff_menu->setItems(diff_menu_entries);
	diff_menu->setCurrentItem(Kpictorial::DiffMedium);		//start with medium difficulty
	difficulty = Kpictorial::DiffMedium;						//sync with intern difficulty
	connect (diff_menu, SIGNAL( activated(int) ), this, SLOT( slotOptionDifficulty(int) ) );

	//the solve and check button, and the undo
	actionCheck = new KAction("&Check", "kpictorial_check.png", "c", this, SLOT( slotCheckPicture() ), actionCollection(), "check");
	actionCheck->setEnabled(false);
	actionSolve = new KAction("&Solve", NULL, 0, this, SLOT( slotSolvePicture() ), actionCollection(), "solve");
	actionSolve->setEnabled(false);
	actionUndo = KStdAction::undo(this, SLOT( slotUndo() ), actionCollection(), "undo");
	actionUndo->setEnabled(false);
	createGUI();

	statusBar()->show();
	//status of tool and statusbar is visible by default
	statusbar_visible = toolbar_visible = true;

	slotNewDocRandom();
	showMaximized();
	//TODO:
	readProperties(config);

}

Kpictorial::~Kpictorial()
{
	delete view;
}

/*
    	This reimplemation catches resizing the window by the user and emits a signal
	to the connected KpictorialView
*/
void Kpictorial::resizeEvent(QResizeEvent * e)
{
//	qDebug("emit resize");
	emit resized();
}

/*
   	create a new picture by random
*/
void Kpictorial::slotNewDocRandom()
{
	int x,y,tmp;

	switch (difficulty)	{
		case Kpictorial::DiffEasy:	x=y=3*5;		break;
		case Kpictorial::DiffMedium:	x=y=4*5;		break;
		case Kpictorial::DiffHard:	x=y=5*5;		break;
		case Kpictorial::DiffHarder:	x=10*5;	y=5*5;	break;
		default: 	//total random
				x = (rand() % 10 + 1) * 5;
				y = (rand() % 5 + 1) * 5;
				if (y > x)	{
					tmp = x;
					x = y;
					y = tmp;
				};	break;
	}

	slotStatusMessage("Creating random");
	delete view;
	view = new KpictorialView(this, x, y);
	setCentralWidget(view);
	connect (this,SIGNAL( resized() ),view,SLOT( slotResized() ));
	connect (view, SIGNAL( signalUndoEmpty(bool) ), this, SLOT( slotUndoEmpty(bool) ));
//	resizeMe(view->size());
	actionCheck->setEnabled(true);
	actionUndo->setEnabled(false);
	actionSolve->setEnabled(true);
	slotStatusMessage(QString("Ready (random picture) size ")+QString::number(view->getPictureX())+QString("x")+ QString::number(view->getPictureY()));
}

/*
   	check if the picture made by the user is the picture he has to make
*/
void Kpictorial::slotCheckPicture()
{
	int result;

	if (view != NULL)
	{
         	result = view->checkPicture();

		switch (result)	{
		case 0:	KMessageBox::information(0,"No, sorry, this is not the picture I want to see");	break;
		case 1:	KMessageBox::information(0, "Yes, you did it, this is the picture I wanted to see :)");
				slotStatusMessage("Solved (by You)");
				actionCheck->setEnabled(false);
				actionSolve->setEnabled(false);
				actionUndo->setEnabled(false);
				break;
		case 2:	KMessageBox::information(0,"You first have to put all pixels in a clear state(no gray ones)"); 	break;
		}

		
	}
}

/*
   	create a new picture from a file
*/
void Kpictorial::slotNewDoc()
{
	KpictorialDoc *doc;

	slotStatusMessage("Opening file");
	//TODO: perhaps an OpenFileDialog without image preview?
  	QString filename = KFileDialog::getOpenFileName(":", "*.xpm *.xbm *.kdf");
	if (!filename.isEmpty())
	{
		doc = new KpictorialDoc(filename);
		if (!doc->statusOk())
		{
			qDebug("oops opening file");
			KMessageBox::detailedError(this,"Error opening file","This can have several reasons, most likely you tried\n\
to open a file in a format I don't like. I like: xpm files the style gimp creates when selecting indexed\n\
color mode using 1bit palette (this makes sense, I only use 2 colors) and xbm files (see help for details).\nYou can see more error\n\
messages when you start me out of the konsole");
			delete doc;
			slotStatusMessage("Ready");
			return;
		}
		else
		{
			delete view;
			view = new KpictorialView(this, doc);
			setCentralWidget(view);
			connect (this,SIGNAL( resized() ),view,SLOT( slotResized() ));
			connect (view, SIGNAL( signalUndoEmpty(bool) ), this, SLOT( slotUndoEmpty(bool) ));
//			resizeMe(view->size());
			actionCheck->setEnabled(true);
			actionUndo->setEnabled(false);
			actionSolve->setEnabled(true);
			slotStatusMessage(QString("Ready (") + filename + QString(") size ") + QString::number(doc->getPictureX()) +QString("x") + QString::number(doc->getPictureY()));
			return;
		}
	}
	slotStatusMessage("Ready");
}

bool Kpictorial::queryExit()	{
	saveProperties(config);
	return true;
}

void Kpictorial::slotQuit()
{
	delete view;
  	exit(0);
}

void Kpictorial::slotSolvePicture()
{
	float result = view->solvePicture();
	slotStatusMessage(QString("Solved (by computer), you had ") + QString::number((double)result, 'g', 2) + QString("% correct"));
	actionCheck->setEnabled(false);
	actionSolve->setEnabled(false);
	actionUndo->setEnabled(false);
}

void Kpictorial::slotStatusMessage(const QString & message)
{
  	statusBar()->message(message);
}

/*
	called whenever the user changes the difficulty settings, and start newRandom Game
*/
void Kpictorial::slotOptionDifficulty(int diff){
//	qDebug("diff got %d", diff);
	difficulty = diff;
	slotNewDocRandom();
}

/*
   	this slot will be called whenever the user wants to save his game
*/
void Kpictorial::slotSaveAs()	{
	
	QString filename = KFileDialog::getSaveFileName(":", "*.kdf", this, NULL);
	if (filename.isEmpty()) return;
	//append .kdf when nessasarry, caseinsensitiv
	if (!filename.contains(".kdf", false)) filename.append(".kdf");
	if (!view->saveAs(filename))	{
			qDebug("oops writing file");
			KMessageBox::detailedError(this,"Error saving file", "Do you have the rights to save the file there?");
	}	

}

void Kpictorial::slotToggleStatusbar()	{

	statusbar_visible = !statusbar_visible;
	if (statusbar_visible) statusBar()->show(); else statusBar()->hide();
}

void Kpictorial::slotToggleToolbar()	{

	toolbar_visible = !toolbar_visible;
	if (toolbar_visible) toolBar()->show(); else toolBar()->hide();

}

void Kpictorial::slotUndo()	{
	view->undo_pop();
}

/*
	connect this to KpictorialView's signal undoEmpty(bool)
*/
void Kpictorial::slotUndoEmpty(bool empty)	{
	actionUndo->setEnabled(!empty);
}
