/***************************************************************************
                          kpictorialdoc.cpp  -  description
                             -------------------
    begin                : Fri Apr 19 2002
    copyright            : (C) 2002 by Roland Lezuo
    email                : roland.lezuo@chello.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kpictorialdoc.h"

/*
   	only decides wich parses to use, XPM or XBM or KDF
*/
KpictorialDoc::KpictorialDoc(QString filename)
{
	picture = NULL;
	userpicture = NULL;

	//be pesimistic :)
	statusok = false;

	//caseinsensitive, try to determit file type
	if (filename.contains(".xpm", false))	{parseXPM(filename);	return;}
	if (filename.contains(".xbm", false))	{parseXBM(filename);	return;}
	if (filename.contains(".kdf", false))	{parseKDF(filename);	return;}
	qDebug("KpictorialDoc::KpictorialDoc could not determite type of file %s\n", filename.latin1());
}


/*
   	this parses KDF (KpictorialDataFile) files
	format description in kpictorialwriter.h
*/
void KpictorialDoc::parseKDF(QString filename)
{
	QFile *file;
	QByteArray content;
	QString *str;
	int point_pos, now_pos, i;
	bool convert_ok;

	file = new QFile(filename);
	if (!file->exists())
	{
         	qDebug("KpictorialDoc::parseKDF file not existing");
		delete file;
		return;
	}
	
	if (!file->open(IO_ReadOnly))
	{
         	qDebug("KpictorialDoc::parseKDF could not open %s for reading", file->name().latin1());
		delete file;
		return;
	}

	content = file->readAll();
	//no need for file any more
	file->close();
	delete file;

	if (content.isNull())
	{
         	qDebug("KpictorialDoc::parseKDF no content");
		return;
	}

	//ok, now parse the content of the file
	now_pos = point_pos = 0;
	//X Size
	point_pos = content.find('.', now_pos);
	if (point_pos == -1)	{
         	qDebug("KpictorialDoc::parseKDF could not find 1st '.'");
		return;
	}
	str = new QString();
	for (i= now_pos; i<point_pos; i++)
		str->append(content.at(i));
	qDebug("KpictorialDoc::parseKDF x %s", str->latin1());

	pictx = convertInt(*str, &convert_ok);
	if (!convert_ok)	{
		qDebug("KpictorialDoc::parseKDF could not convert x");
		return;
	}
	delete str;
	now_pos = point_pos + 1;
	//Y size
	point_pos = content.find('.', now_pos);
	if (point_pos == -1)	{
         	qDebug("KpictorialDoc::parseKDF could not find 2nd '.'");
		return;
	}
	str = new QString();
	for (i= now_pos; i<point_pos; i++)
		str->append(content.at(i));
	qDebug("KpictorialDoc::parseKDF y %s", str->latin1());

	picty = convertInt(*str, &convert_ok);
	if (!convert_ok)	{
		qDebug("KpictorialDoc::parseKDF could not convert y");
		return;
	}
	delete str;
	now_pos = point_pos + 1;

	//FIXME: should not realy rely that kdf files have "nice" x and y dimensions
	//FIXME: could have also been made by different programms

	qDebug("KpictorialDoc::parseKDF size %dx%d", pictx, picty);
	//the real picture
	point_pos = content.find('.', now_pos);
	if (point_pos == -1)	{
         	qDebug("KpictorialDoc::parseKDF could not find 3rd '.'");
		return;
	}
	if (point_pos - now_pos != pictx * picty)	{
		qDebug("KpictorialDoc::parseKDF picture wrong size");
		return;
	}
	//get the memory
	picture = calloc(pictx*picty, sizeof(char));
	//and fill it
	for (i=now_pos; i<point_pos; i++)	{
       	if (content.at(i) == '1') *((char*)picture + i - now_pos) = 1;
	}
	now_pos = point_pos + 1;

	//the user picture
	point_pos = content.find('.', now_pos);
	if (point_pos == -1)	{
         	qDebug("KpictorialDoc::parseKDF could not find 4th '.'");
		return;
	}
	if (point_pos - now_pos != pictx * picty)	{
		qDebug("KpictorialDoc::parseKDF userpicture wrong size");
		return;
	}
	userpicture = calloc(pictx*picty, sizeof(char));
	//and fill it
	for (i=now_pos; i<point_pos; i++)	{
       	if (content.at(i) == '1') *((char*)userpicture + i - now_pos) = 1;
       	if (content.at(i) == '2') *((char*)userpicture + i - now_pos) = 2;
	}
	statusok = true;
}


/*
	this parses an xbm black & white file
*/

void KpictorialDoc::parseXBM(QString filename)
{
	QFile *file;
	QByteArray content;
	QString *definestr, *contentstr;
	QStringList definesplit, contentsplit;
	QString split[3];
	int i, splitcounter, cross_pos, newline_pos;
	int pictrealx, pictrealy;
	bool convert_ok;

	file = new QFile(filename);
	if (!file->exists())
	{
         	qDebug("KpictorialDoc::parseXBM file not existing");
		delete file;
		return;
	}
	
	if (!file->open(IO_ReadOnly))
	{
         	qDebug("KpictorialDoc::parseXBM could not open %s for reading", file->name().latin1());
		delete file;
		return;
	}

	content = file->readAll();
	//no need for file any more
	file->close();
	delete file;

	if (content.isNull())
	{
         	qDebug("KpictorialDoc::parseXBM no content");
		return;
	}

	//ok, now parse the content of the file
	//the files have 2 #defines: #define XXX_width X, #define XXX_height Y
	//get them
	/*
          			THE FIRST #DEFINE

	*/	
	pictrealx = pictrealy = -1;		//init with illegal values
	
	cross_pos = content.find('#', 0);	
	if (cross_pos == -1)	{
		qDebug("KpictorialDoc::parseXBM did not find 1st #define");
		return;
	}
	//ok, cross_pos now points to the first #define
	newline_pos = content.find('\n', cross_pos);
	if (newline_pos == -1)	{
		qDebug("KpictorialDoc::parseXBM did not find end of 1st #define");
		return;
	}
	//and newline_pos points to the end of the first #define
	//get this line into a QString class
	definestr= new QString();
	for (i=cross_pos; i<newline_pos; i++)	{
		definestr->append(content.at(i));
	}
	qDebug("KpictorialDoc::parseXBM 1st #define: %s", definestr->latin1());

	//split the string at it whitespaces, but don't allow whitespace only splits
	definesplit = QStringList::split(QChar(' '), *definestr, false);
	delete definestr;		//in case we retuns early through errors
	splitcounter = 0;
	for (QStringList::Iterator it = definesplit.begin(); it != definesplit.end(); ++it)	{
		//ok, the first should be the #define, the second the name, the third the widht/height
		if (splitcounter < 3) split[splitcounter++] = *it;
		else	{
			qDebug("KpictorialDoc::parseXBM 1st #define splits into more than 3");
			return;
		}
	}
	if (splitcounter != 3)	{
		qDebug("KpictorialDoc::parseXBM 1st #define splits not into 3");
		return;
	}
	qDebug("KpictorialDoc::parseXBM 1st #define: %s,%s,%s", split[0].latin1(), split[1].latin1(), split[2].latin1());
	
	//convert to int
	convert_ok = false;
	if (split[1].contains("_width"))	{
		pictrealx = convertInt(split[2],&convert_ok);
	}	
	else if (split[1].contains("_height"))	{
		pictrealy = convertInt(split[2],&convert_ok);
	}	

	//did it work?
	if (!convert_ok)	{
		qDebug("KpictorialDoc::parseXBM 1st #define: could not convert to int");
		return;
	}
	//either xmax, or ymax are no converted, check this later, now proceed to the next #define
	/*
          			THE SECOND #DEFINE

	*/	
	//now search from newline_pos, to find the second #define
	cross_pos = content.find('#', newline_pos);	
	if (cross_pos == -1)	{
		qDebug("KpictorialDoc::parseXBM did not find 2st #define");
		return;
	}
	//ok, cross_pos now points to the 2nd #define
	newline_pos = content.find('\n', cross_pos);
	if (newline_pos == -1)	{
		qDebug("KpictorialDoc::parseXBM did not find end of 2nd #define");
		return;
	}
	//and newline_pos points to the end of the first #define
	//get this line into a QString class
	definestr= new QString();
	for (i=cross_pos; i<newline_pos; i++)	{
		definestr->append(content.at(i));
	}
	qDebug("KpictorialDoc::parseXBM 2nd #define: %s", definestr->latin1());

	//split the string at it whitespaces, but don't allow whitespace only splits
	definesplit = QStringList::split(QChar(' '), *definestr, false);
	delete definestr;		//in case we retuns early through errors
	splitcounter = 0;
	for (QStringList::Iterator it = definesplit.begin(); it != definesplit.end(); ++it)	{
		//ok, the first should be the #define, the second the name, the third the widht/height
		if (splitcounter < 3) split[splitcounter++] = *it;
		else	{
			qDebug("KpictorialDoc::parseXBM 2nd #define splits into more than 3");
			return;
		}
	}
	if (splitcounter != 3)	{
		qDebug("KpictorialDoc::parseXBM 2nd #define splits not into 3");
		return;
	}
	qDebug("KpictorialDoc::parseXBM 2nd #define: %s,%s,%s", split[0].latin1(), split[1].latin1(), split[2].latin1());
	
	//convert to int
	convert_ok = false;
	if (split[1].contains("_width"))	{
		pictrealx = convertInt(split[2], &convert_ok);
	}	
	else if (split[1].contains("_height"))	{
		pictrealy = convertInt(split[2], &convert_ok);
	}	

	//did it work?
	if (!convert_ok)	{
		qDebug("KpictorialDoc::parseXBM 2nd #define: could not convert to int");
		return;
	}

	/*
          		END #DEFINES
	*/
	//ok, now both, xmax and ymax should be not -1 and greater 0
       if ((pictrealx <= 0) || (pictrealy <= 0))	{
		qDebug("KpictorialDoc::parseXBM wrong dimensions: %dx%d", pictrealx, pictrealy);
		return;
	}	
	qDebug("KpictorialDoc::parseXBM dimensions: %dx%d", pictrealx, pictrealy);

	//ok the size of the picture seems to be clear, no go over to the content
	//which is between { and };

	int open_bracket, close_bracket;
	//find the opening bracket from last known position in file
	open_bracket = content.find('{', newline_pos);
       if (open_bracket == -1)	{
		qDebug("KpictorialDoc::parseXBM could not find {");
		return;
	}	
	close_bracket = content.find('}', open_bracket);
       if (close_bracket == -1)	{
		qDebug("KpictorialDoc::parseXBM could not find }");
		return;
	}	
	//we have the brackets, start parsing content
	//this realy shoundn't be
	if (!(open_bracket < close_bracket))	{
		qDebug("KpictorialDoc::parseXBM } before {");
		return;
	}	

	//get the content into a String
	contentstr = new QString();
	for (i=open_bracket +1; i<close_bracket; i++)	{
		contentstr->append(content.at(i));
	}
	//and split every ',' but don't allow whitespace only splits
	contentsplit = QStringList::split(QChar(','), *contentstr, false);
	delete contentstr;		//no need for that any more

	//get the mem for the picture
	//if pict* % 5 != 0 than blow the picture up so it fills the 5x5 squares
	//this is done by adding white pixels to the right and at the bottom
	if (pictrealx % 5) pictx = pictrealx + 5 - pictrealx % 5; else pictx = pictrealx;
	if (pictrealy % 5) picty = pictrealy + 5 - pictrealy % 5; else picty = pictrealy;
	picture = calloc(pictx*picty, sizeof(char));

	splitcounter = 0;
	QString to_convert;
//	int convert_base;
	unsigned int converted;
	unsigned int bitmask;

	for (QStringList::Iterator it = contentsplit.begin(); it != contentsplit.end(); ++it)	{
		//okay, in here parsing and converting the colors to total black or white is done
		//rule: every color <=0x80 is white, everyting else is black
		//QString::toInt doesnot take 0xFF only FF as hexvalues, so one more split :(


		/*
		//this detects x and X in spilt --> a hex number
		if ((*it).contains('x', false))	{
			//replace the x caseinsestive with a 0
			to_convert = (*it).replace(QRegExp("x",false), "0");
			convert_base = 16;
		}
		else	{
			//the value seems to be decimal
			convert_base = 10;
			to_convert = *it;
		}

		converted = to_convert.toUInt(&convert_ok, convert_base);*/
		converted = convertInt(*it, &convert_ok);
		if (!convert_ok)	{
			qDebug("KpictorialDoc::parseXBM could not convert image data %s", (*it).latin1());
			return;
		}

		//prevent any overflows
		if (splitcounter > pictx*picty)	{
			qDebug("KpictorialDoc::parseXBM overflowing: image is bigger as defined!");
			return;
		}
	
		//ok, now extract every bit and set corespondig pixels in pixmap
		bitmask = 0x01;
		for (i=0; i<8; i++)	{
//			qDebug("bm: %x, conv: %x, bm&&conv: %x", bitmask, converted, bitmask&converted);
			if (bitmask & converted)	{
				//this is non trivial, because we could have white borders, to fill the 5x5 squares!
				*((char *)picture + (splitcounter / pictrealx) * pictx + splitcounter % pictrealx) =1;
			}
			splitcounter ++;
			//if a byte if filled a little bit
			if (splitcounter % pictrealx == 0) break;
			bitmask = bitmask << 1;
		}
	}

	if (splitcounter != pictrealx*pictrealy)	{
		qDebug("KpictorialDoc::parseXBM picture size (%d) differs from expected size (%d)", splitcounter, pictrealx*pictrealy);
		return;
	}

	//unbeliveable, we are still here, after that enourmous number of returns :)
	statusok = true;
	
	//this format does not support userpicture status info, so everything is gray
	userpicture = calloc(pictx*picty, sizeof(char));
	for (i=0; i<pictx*picty; i++)
		*((char*)userpicture + i) = 2;	
	
}


/*
	tries to open an 1-bit xpm file, like created by gimp
	I think I fixed all steadily increasing counters when searching for a special char in the file, so
	no endless loop should be possible any more (famous last words?) but did not do that very exact, because
	QByteArray.at() called with index out of bounds doesn't realy harm (expect some output that it is out of bounds)
	but at some point the overflow should be recognized and we return
*/
void KpictorialDoc::parseXPM (QString filename)
{
	QFile *file;
	QByteArray content;
	int i;
	bool in_open_quot;
	bool parsing_done;
	int section;
	int header_data[4];	//contains: x,y,#colors,#pictures
	int x,y;
	
	file = new QFile(filename);
	if (!file->exists())
	{
         	qDebug("KpictorialDoc::parseXPM file not existing");
		delete file;
		return;
	}
	
	if (!file->open(IO_ReadOnly))
	{
         	qDebug("KpictorialDoc::parseXPM could not open %s for reading", file->name().latin1());
		delete file;
		return;
	}

	content = file->readAll();
	//no need for file any more
	file->close();
	delete file;

	if (content.isNull())
	{
         	qDebug("KpictorialDoc::parseXPM no content");
		return;
	}

	//ok, now parse the content of the file
	//what is a section: everything between two "s
	//section: 0 until first opening " found and then set to 1
	//1 means: header informations about the picture
	//2,3,4 means: color declarations
	//everything else: the picture lines
	section = 0;
	in_open_quot = false;
	parsing_done = false;
	i = 0;	//byte position in array
	y = 0;	//picture lines read counter

	bool ok;
	int header_data_counter, contentsize;
	QString *header;
	QStringList split_header;


	contentsize = content.size();
	while (!parsing_done)
	{
		if (!in_open_quot)
		{
			while (((char)content.at(i)!='"')  && (i<contentsize)) i++;
			//just found an opening ", or found none and overflowed
			if (i >= contentsize)	{
				qDebug("KpictorialDoc::parseXPM searching open quot overflowed");
				return;
			}

			in_open_quot = true;
			section ++;
			qDebug("KpictorialDoc::parseXPM: entered section %d at offset 0x%X", section, i);
		}
		else
		{
			//when entering here, we have just entered a new section
			switch (section) {
			case 1:
				//we are in the header section now
				header = new QString();
				while ((content.at(++i) != '"') && (i<contentsize)) header->append(content.at(i));
				if (i >= contentsize)	{
					qDebug("KpictorialDoc::parseXPM searching header overflowed");
					return;
				}

				qDebug("Kpictorial::parseXPM header is %s", header->latin1());
				//now i points to the closing ", all date between " and " are in header now.
				split_header = QStringList::split(QString(" "), *header);
       			//no need for header any more
				delete header;

				header_data_counter = 0;
				for (QStringList::Iterator it = split_header.begin(); it != split_header.end(); ++it)
				{
					if (header_data_counter == 4) {
						qDebug("KpictorialDoc::parseXPM too many fields in header");
						return;
					}
					ok = false;
					header_data[header_data_counter++] = convertInt(*it, &ok);
					if (!ok) {
                                     	qDebug("KpictorialDoc::parseXPM error converting header_date [%d] : %s", header_data_counter - 1, (*it).latin1());
						return;
					}
				}

				if (header_data_counter != 4) {
					qDebug("KpictorialDoc::parseXPM not 4 values in header");
					return;
				}
				//ok, we are here, now check if we like that picture, this means:
				//no dimension may be less than 0, 3 colors, and 1 picture in it
				if (header_data[0] <= 0) {
					qDebug("KpictorialDoc::parseXPM picutre's x is invalid(%d)", header_data[0]);
					return;
				}
				if (header_data[1] <= 0) {
					qDebug("KpictorialDoc::parseXPM picutre's y is invalid(%d)", header_data[1]);
					return;
				}
				if (header_data[2] != 3) {
					qDebug("KpictorialDoc::parseXPM picutre's #colors is invalid(%d)", header_data[2]);
					return;
				}
				if (header_data[3] != 1) {
					qDebug("KpictorialDoc::parseXPM picutre's #number is invalid(%d)", header_data[3]);
					return;
				}

				i++;
				in_open_quot=false;
				//its time to get some memory for the picture and take over the values
				int pictrealx, pictrealy;		//because be blow the up when nesessary to fill 5x5 squares
				pictrealx = header_data[0];
				pictrealy = header_data[1];
		
				//lets see if the file size isn'g too small for this picture
				if (content.size() < i + pictrealx*pictrealy)	{
					qDebug("KpictorialDoc::parseXPM file size (%d) is less than needed (%d)",content.size(), i+pictrealy*pictrealx);
					return;
				}
				//be blow up the picture to fit 5x5 squares if nesessary by adding white pixels left and bottom
				if (pictrealx % 5) pictx = pictrealx + 5 - pictrealx % 5; else pictx = pictrealx;
				if (pictrealy % 5) picty = pictrealy + 5 - pictrealy % 5; else picty = pictrealy;
				picture = calloc(pictx*picty, sizeof(char));
				break;
			case 2:
			case 3:
			case 4:
				//this are just the color declarations, we don't care and asume, that . is white and + is black
				while ((content.at(++i) != '"') && (i<contentsize));
				if (i >= contentsize)	{
					qDebug("KpictorialDoc::parseXPM skipping color declarations overflowed");
					return;
				}
				i++;
				in_open_quot=false;
				qDebug("KPictorialDoc::parseXPM skipped color declaration in section %d", section);
				break;
			default:
				//these are all the picture lines
				qDebug("KpictorialDoc::parseXPM reading picture line %d of %d", section - 4, pictrealy);
				//i is pointing to the beginning " of this line
				x = 0;
				while (content.at(++i) != '"') {
					if (content.at(i) == '+') *((char*)picture + x +y*pictx) = (char)1;
					x++;
					if (x > pictrealx)	{
						qDebug("KpictorialDoc::parseXPM overflowing line %d" , section -4);
						return;
					}
				}
				if (x != pictrealx) {
					qDebug("KpictorialDoc::parseXPM picture line %d has size %d instead of %d", section -4, x, pictrealx);
					return;
				}
				i++;
				in_open_quot=false;
				if (++y == pictrealy) parsing_done = true;	//we did it!
			}	//switch
		}      //else
	}	//while
	//mark this instance as valid
	statusok = true;

	//this format does not support userpicture status info, so everything is gray
	userpicture = calloc(pictx*picty, sizeof(char));
	for (i=0; i<pictx*picty; i++)
		*((char*)userpicture + i) = 2;	
}

/*
 	this convert a string to an integer, QString may be hex 0x.. 0X... or decimal, bool is true when convert is ok
*/
unsigned int KpictorialDoc::convertInt(QString in_str, bool *out_ok)	{

	int convert_base;

	//this detects x and X in in_str --> a hex number
	if (in_str.contains('x', false))	{
		//replace the x caseinsestive with a 0
		in_str = in_str.replace(QRegExp("x",false), "0");
		convert_base = 16;
	}
	else	{
		//the value seems to be decimal
		convert_base = 10;
	}

	return in_str.toUInt(out_ok, convert_base);
}

/*
   	This constructor creates a new random picture
*/
KpictorialDoc::KpictorialDoc(int x, int y)
{
	int i;
	int j;

	pictx = x;
	picty = y;
	picture = calloc(x*y, sizeof(char));
	userpicture = calloc(x*y, sizeof(char));

	j = rand() % 4 + 2;

	for (i=0; i<x*y; i++) {
		if ((char)(rand() % j)) *((char*)picture + i) = 1;
		else *((char*)picture + i) = 0;
		*((char*)userpicture + i) = 2;
	}
	statusok = true;
}

/*
   	free memory we used
*/
KpictorialDoc::~KpictorialDoc()
{
	free(picture);
	free(userpicture);
}


/*
   	returns true when an valid pictures is in here
*/
bool KpictorialDoc::statusOk()
{
  	return statusok;
}

/*
	returns size of picture in x-axis
*/
int KpictorialDoc::getPictureX()
{
	return pictx;
}

/*
	size of picture in y-axis
*/
int KpictorialDoc::getPictureY()
{
	return picty;
}

/*
	returns a void* field containing the picture
	should be interpreted as char * where
	a 1 means that this point should be set
*/
const void * KpictorialDoc::getPicture()
{
	return picture;
}

/*
	returns the string describing continous blocks along y-axis
	in line "index", null when index out of bounds
*/
QString* KpictorialDoc::getPictureInfoX(int index)
{
	QString *to_return;

	if ((index < 0) || (index >= pictx)) return NULL;

	to_return = new QString();
	int continous = 0;
	int x = index;

	while (x < pictx*picty)
	{
       	if (*((char*)picture + x) == 1)
		{
                	continous ++;
		}
		else
		{
                 	if (continous)
			{
				//because 2 number greater than 10 in two columns next to each other would overlap
				if (continous >= 10)
				{
                              	if (!to_return->isEmpty()) to_return->append("\n\n.");
					else to_return->append(".");
					//create a split at every number
					QStringList str = QStringList::split("", QString::number(continous));
					for (QStringList::Iterator it=str.begin(); it != str.end(); ++it)
					{
                                     	to_return->append(*it);
						to_return->append("\n");
					}
					continous = 0;
				}
				else
				{
					if (!to_return->isEmpty()) to_return->append("\n");
	                       	to_return->append(QString::number(continous));
					//to_return->append("\n");
					continous = 0;
				}
			}
		}
		x += pictx;
	}
      	if (continous)
		{
			if (continous >= 10)
			{
                     	if (!to_return->isEmpty()) to_return->append("\n\n.");
				else to_return->append(".");
				//create a split at every number
				QStringList str = QStringList::split("", QString::number(continous));
				for (QStringList::Iterator it=str.begin(); it != str.end(); it++)
				{
                            	to_return->append(*it);
					to_return->append("\n");
				}
				continous = 0;
			}
			else
			{
				if (!to_return->isEmpty()) to_return->append("\n");
                     	to_return->append(QString::number(continous));
				//to_return->append("\n");
				continous = 0;
			}
		}

	if (to_return->isEmpty()) to_return->append("0");
	return to_return;	
}

/*
	returns the string describing continouse blocks along x-axis
	in the row number "index", null when index out of bounds
*/
QString* KpictorialDoc::getPictureInfoY(int index)
{
	QString *to_return;
	QStringList split;

	if ((index <0) || (index >= picty)) return NULL;

	to_return = new QString();
	int continous = 0;
	int y = index * pictx;

	while (y < pictx*index + pictx)
	{
       	if (*((char*)picture + y) == 1)
		{
                	continous ++;
		}
		else
		{
                 	if (continous)
			{
                       	to_return->append(QString::number(continous));
				to_return->append(" ");
				continous = 0;
			}
		}
		y ++;
	}
	if (continous)
	{
		to_return->append(QString::number(continous));
		to_return->append(" ");
	}

	if (to_return->isEmpty()) to_return->append("0 ");
	return to_return;
}

/*
	this methode checks wheter den input field an the
	field represented by this class are equal, the size
	of the pointer to check with must be the same size than the picture in here
*/
bool KpictorialDoc::controlPicture(const void *pict)
{
	int i;

	for (i=0; i<pictx*picty; i++)
	{
		//qDebug("%d...%d:%d",i,*((char*)picture + i), *((char*)pict + i));
         	if ( (char)*((char*)picture + i) != (char)*((char*)pict + i) ) return false;
	}
	return true;
}

/*
   	this routine (not very efficient, is it NP-complete?) checks wheter a picture is definite
	and returns the number of found possible solutions
*/
int KpictorialDoc::isDefinite()
{
	return 0;
}

void * KpictorialDoc::getUserPicture()	{
  	return userpicture;
}



//writes a file in kdf file format which is defined as follows:
/*
	file format: (pretty baindead)
		sizeX.sizeY.pict.upict.
       no whitespaces allowed
	all numbers in decimal format
	upict and pict are of size sizeX * sizeY
	pict is made of 0 and 1
	upict of 0, 1 and 2
	where 0...white pixel, 1...black pixel, 2...gray pixel
*/
bool KpictorialDoc::saveAs(QString name)	{

	QFile *file = new QFile(name);
	QString str;
	int i;

	//we simple overwrite
	if (!file->open(IO_WriteOnly | IO_Truncate))	{
		qDebug("KpictorialDocWriter::write can't open for writing %s", name.latin1());
		delete file;
		return false;
	}
	str = QString::number(pictx);
	file->writeBlock(str.latin1(), str.length());
	file->putch('.');
	str = QString::number(picty);
	file->writeBlock(str.latin1(), str.length());
	file->putch('.');
	for (i=0; i<pictx*picty; i++)
		file->putch( *((char*)picture +i) + '0' );	
	file->putch('.');
	for (i=0; i<pictx*picty; i++)
		file->putch( *((char*)userpicture +i) + '0' );	
	file->putch('.');
	file->close();
	delete file;
	return true;
}
