/***************************************************************************
                          kpictorialview.cpp  -  description
                             -------------------
    begin                : Thu Apr 18 2002
    copyright            : (C) 2002 by Roland Lezuo
    email                : roland.lezuo@chello.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kmainwindow.h"
#include "kpictorial.h"
#include "kpictorialview.h"
#include "kpictorialview.moc"

/*
	this creates a new picture from file, file will be deleted in KpictorialView, doc must
	already be initialized
*/
KpictorialView::KpictorialView(QWidget *parent, KpictorialDoc *init_doc) : QCanvasView(parent)
{
	doc = init_doc;
	init();
}


/*
	creates a random picture in size x*y
*/
KpictorialView::KpictorialView(QWidget *parent, int x, int y) : QCanvasView(parent)
{
	//create a random picture
	doc = new KpictorialDoc(x,y);
	init();
}

/*
   	this is what both constructors do (everything expect the document init)
*/
void KpictorialView::init()
{
	canvas = new QCanvas  (60,60);
	setCanvas(canvas);

	pixsize = 13;
	textsize = 10;
	bwhite = new QBrush(QColor(0xFF,0xFF,0xFF));
	bblack = new QBrush(QColor(0x00,0x00,0x00));
	gray  = new QColor(0xA0,0xA0,0xA0);
	bgray = new QBrush(QColor(0xD0,0xD0,0xD0));
	red = new QColor(0xC0,0x00,0x00);
	green = new QColor(0x00,0xC0,0x00);

	xmax = doc->getPictureX();
	ymax = doc->getPictureY();

	//this will be freed in doc
	picture = doc->getUserPicture();
	
	//still not solved, so take clicks
	solved = false;	

	//init undo
	undo_flush();

	createPicture();
	show();
}


KpictorialView::~KpictorialView()
{
	delete canvas;
	delete doc;
	delete bwhite;
	delete bblack;
	delete bgray;
	delete gray;
	delete green;
	delete red;
}


/*
	this draws all the initial state, the numbers, the lines...
*/
void KpictorialView::createPicture()
{
	int x, y;
	QCanvasRectangle *rect;
	QCanvasText *text;
	QFont *textfont  = new QFont();
	textfont->setPixelSize(pixsize);

//	qDebug("x(0): %s", doc->getPictureInfoX(0)->latin1());
//	qDebug("y(0): %s", doc->getPictureInfoY(0)->latin1());

	//first create the labels for rows and lines of the picture
	x_off = y_off = 0;

	//calculate x_off
	for (y=0; y<ymax; y++)
	{
		text = new QCanvasText(*doc->getPictureInfoY(y), *textfont, canvas);
		if (x_off < text->boundingRect().right()) x_off = text->boundingRect().right();
		delete text;
	}
	//calculate y_off
	for (x=0; x<xmax; x++)
	{
		text = new QCanvasText(*doc->getPictureInfoX(x), *textfont, canvas);
		if (y_off < text->boundingRect().bottom()) y_off = text->boundingRect().bottom();
		delete text;
	}

	//the labels for the columns (in y-axis direction)
	for (x=0; x<xmax; x++)
	{
		//above the picture
		text = new QCanvasText(*doc->getPictureInfoX(x), *textfont,  canvas);
		//thanks to Mike Bennett for this style tip
		text->setX(x_off + (x+1)*pixsize -  text->boundingRect().width());
		text->setTextFlags(Qt::AlignRight);
		text->setY(y_off - text->boundingRect().bottom());
		text ->setFont(*textfont);
		text->show();

		//below the picture
		text = new QCanvasText(*doc->getPictureInfoX(x), *textfont,  canvas);
		text->setX(x_off + (x+1)*pixsize -  text->boundingRect().width());
		text->setTextFlags(Qt::AlignRight);
		text->setY(y_off + ymax * pixsize);
		text ->setFont(*textfont);
		text->show();

	}

	for (y=0; y<ymax; y++)
	{
		//left from the picture
		text = new QCanvasText(*doc->getPictureInfoY(y), *textfont, canvas);
		text->setY(y_off + y*pixsize);
		text->setX(x_off - text->boundingRect().right());
		text->show();

		//right from the picture
		text = new QCanvasText(*doc->getPictureInfoY(y), *textfont, canvas);
		text->setY(y_off + y*pixsize);
		text->setX(x_off + xmax * pixsize);
		text->show();
	}


//	const void* pict = doc->getPicture();

	//framed field
	//top
	line = new QCanvasLine(canvas);
	line->setPoints(x_off -1, y_off -1, x_off + xmax*pixsize -1, y_off -1);
	line->setPen(QPen(*gray));
	line->show();

	//bottom
	line = new QCanvasLine(canvas);
	line->setPoints(x_off  -1, y_off + ymax*pixsize -1, x_off + xmax*pixsize -1, y_off + ymax*pixsize -1);
	line->setPen(QPen(*gray));
	line->show();

	//left
	line = new QCanvasLine(canvas);
	line->setPoints(x_off  -1, y_off -1, x_off  -1, y_off + ymax*pixsize -1);
	line->setPen(QPen(*gray));
	line->show();

	//right
	line = new QCanvasLine(canvas);
	line->setPoints(x_off + xmax*pixsize -1, y_off -1, x_off + xmax*pixsize -1, y_off + ymax*pixsize -1);
	line->setPen(QPen(*gray));
	line->show();

	//seperate every 5 lines
	for (x=0; x<xmax/5; x++)	{
		line = new QCanvasLine(canvas);
		line->setPoints(x_off + 5*(x+1)*pixsize -1, y_off, x_off + 5*(x+ 1)*pixsize - 1, y_off + ymax*pixsize - 2);
		line->setPen(QPen(*gray));
		line->show();
	}

	for (y=0; y<ymax/5; y++)	{
		line = new QCanvasLine(canvas);
		line->setPoints(x_off , y_off + 5*(y+1)*pixsize - 1 , x_off + xmax*pixsize - 2, y_off + 5*(y+1)*pixsize - 1);
		line->setPen(QPen(*gray));
		line->show();
	}
	

	//then create the picture
	for (x=0; x<xmax; x++)
		for (y=0; y<ymax; y++)
		{
			rect = new QCanvasRectangle(x_off + x*pixsize, y_off + y*pixsize, pixsize-1, pixsize-1, canvas);
			
			//would be to easy, not?
			/*
	              if (*((char*)pict + x + xmax*y))
				rect->setBrush(*black);
			else
				rect->setBrush(*white);
			*/
			//this is better
			switch (*((char*)picture + x + xmax*y))	{
				case 0:	rect->setBrush(*bwhite);	break;
				case 1:	rect->setBrush(*bblack);	break;
				case 2:	rect->setBrush(*bgray);	break;
			}
			rect->show();
		}
	canvas->resize(xmax * pixsize +  2 * x_off , ymax * pixsize + 2* y_off );
	delete textfont;
}

/*
   	call this when parent is resized, this resizes the view
*/
void KpictorialView::slotResized()	{
	resize( ((Kpictorial *)(this->parent()))->centralWidget()->size()  );
}

/*
   	reimplemantation: change color of pixel and coresponding logical structure when clicked on a pixel
*/
void KpictorialView::mousePressEvent(QMouseEvent *e)
{
	int x_abs, y_abs;

//	qDebug("click");

	//don't take clicks, when solved
	if (solved) return;

	x_abs = e->x() + contentsX();
	y_abs = e->y() + contentsY();

	//have to convert absolut coords to logical pixel coords, call only with mousebuttons assigned to a function!
	if (e->button() == Qt::RightButton)	pixelHitUndo((x_abs - x_off) / pixsize, (y_abs - y_off) / pixsize, Qt::RightButton);
	if (e->button() == Qt::LeftButton)	pixelHitUndo((x_abs - x_off) / pixsize, (y_abs - y_off) / pixsize, Qt::LeftButton);

	canvas->update();
}

/*
   	call this to controll if the picture made by the user is the picture to be constructed
	returns: 	0, not the correct picture
			1, the correct one
			2, there are still gray field
*/
int KpictorialView::checkPicture()
{
	//are there any gray fields?
	for (int i = 0; i<xmax*ymax; i++){
		if (*((char*)picture + i) == 2)	{
		  	return 2;
		}
	}

	if (doc->controlPicture(picture)) 	{
		solved = true;		//picture solved, don't take clicks any more
		return 1;
	}

	return 0;
}

/*
   	reimplemantation of size, this is the size of the canvas + some extra space to avoid the scrollbars
*/
QSize KpictorialView::size()	{
  	return canvas->size() + QSize(50,50);
}

/*
   	this solves the picture for the user, and retruns percentage of correctness
*/
float KpictorialView::solvePicture()
{
	//get the original from the document
	const void* pict = doc->getPicture();
	QCanvasItemList list;
	QCanvasRectangle *rect;
	int x,y, correct = 0;

	for (x=0; x<xmax; x++)
		for (y=0; y<ymax; y++)
		{
			//only when user has decided, gray ones will be left out
			if (*((char*)picture + x + y*xmax) != 2)	{
				//this rectangle will be green or red depending wheter the pixel is correct guessed by user
				rect = new QCanvasRectangle(x_off + x*pixsize + pixsize/2 - 2,y_off + y*pixsize + pixsize/2 - 2,4,4,canvas);
				rect->setZ(100);

				if (*((char*)pict + x + y*xmax) == 1)
				{
					//get Pixel from canvas and set correct color
					list = canvas->collisions( QPoint ( x*pixsize + pixsize/2 + x_off, y*pixsize + pixsize/2 + y_off) );
					QCanvasItemList::Iterator it = list.begin();	
					((QCanvasRectangle*)(*it))->setBrush(*bblack);
					//has the user set it to this color?, when gray, then don't set the gren or red point
					if (*((char*)picture + x+ y*xmax) == *((char*)pict + x + y*xmax)) 	{
						rect->setPen(*green);
						rect->setBrush(QBrush(*green));
						correct ++;
					}
					else	{
						rect->setPen(*red);
						rect->setBrush(QBrush(*red));
					}
					//*((char*)picture + x + y*xmax) = 1;
				}
				else
				{
					list = canvas->collisions( QPoint ( x*pixsize + pixsize/2  + x_off, y*pixsize + pixsize/2  + y_off) );
					QCanvasItemList::Iterator it = list.begin();	
					((QCanvasRectangle*)(*it))->setBrush(*bwhite);
					if (*((char*)picture + x+ y*xmax) == *((char*)pict + x + y*xmax)) 	{
						rect->setPen(*green);
						rect->setBrush(QBrush(*green));
						correct ++;
					}
					else	{
						rect->setPen(*red);
						rect->setBrush(QBrush(*red));
					}
					//*((char*)picture + x + y*xmax) = 0;
				}
				rect->show();
			}//endif user has decided, just show what it would have been
			else
			{
				if (*((char*)pict + x + y*xmax) == 1)
				{
					//get Pixel from canvas and set correct color
					list = canvas->collisions( QPoint ( x*pixsize + pixsize/2 + x_off, y*pixsize + pixsize/2 + y_off) );
					QCanvasItemList::Iterator it = list.begin();	
					((QCanvasRectangle*)(*it))->setBrush(*bblack);
				}
				else
				{
					list = canvas->collisions( QPoint ( x*pixsize + pixsize/2  + x_off, y*pixsize + pixsize/2  + y_off) );
					QCanvasItemList::Iterator it = list.begin();	
					((QCanvasRectangle*)(*it))->setBrush(*bwhite);
				}
			}
		}
	canvas->update();
	solved = true;			//it is solved now, we don't take clicks any more
	return (((float)correct) * 100)/ (float)(xmax*ymax);
}

/*
   	returns size of picture in X Axis direction
*/
int KpictorialView::getPictureX()	{
  	return xmax;
}

/*
   	returns size of picture in Y Axis direction
*/
int KpictorialView::getPictureY()	{
  	return ymax;
}

/*
	returns a pointer containig the picture, treat as char* 0...white, 1...black
*/
const void *KpictorialView::getPicture()	{
  	return doc->getPicture();
}


/*
	returns a pointer containig the initial picture the user guessed
	this are all 2's for xpm and xbm files, but may contain everything for
	kdf files. The memory MUST not be freed by the caller
*/
const void *KpictorialView::getUserPicture()	{
  	return picture;
}

/*
	logical number of pixel and MouseButton which were clicked on it
*/
void KpictorialView::undo_push(int x, int y, Qt::ButtonState butt)	{
	undo_last++;					//next position in array
	undo_last %= UNDO_SIZE; 		//zyclic

	if (undo_last == undo_first)	{	//array is full
		undo_first ++;			//overwite oldest entry
		undo_first %= UNDO_SIZE;	//zyclic
	}
	//we save negativ values when hit with right mouse button
	//and because 0 can't be signed we also add an offset
	if (butt == Qt::RightButton) undo[undo_last] = -(x+ y*xmax) - 1;	//and save value
	if (butt == Qt::LeftButton) undo[undo_last] = x+ y*xmax + 1;	//and save value
//	qDebug("KpictorialView::undo_push pushed %d state %d:%d", undo[undo_last], undo_first, undo_last);


	//emit only when the first element is in the structure
	if ((undo_last - undo_first == 1) || ((undo_last == 0)  && (undo_first== UNDO_SIZE - 1))) emit signalUndoEmpty(false);
}

/*
	undo the last click
*/
void KpictorialView::undo_pop()	{

	int pixel_restore, pixel;
	int pixel_x_log, pixel_y_log;


	if (undo_last == undo_first)	return;	//array is empty, there is nothing to do

	pixel_restore = undo[undo_last];
	undo_last --;						//remove last object from array
	if (undo_last == -1)	{
		undo_last += UNDO_SIZE;		//zyclic
	}

	//signal that the structure is empty now
	if (undo_last == undo_first) emit signalUndoEmpty(true);

	//restore the saved state of the pixel
	if (pixel_restore == 0)	{
		qDebug("KpictorialView::undo_pop Invalid undo entry (0)");
		return;
	}

	//because 0 is forbidden (can't be signed) remove offset from pixel
	if (pixel_restore < 0) pixel = -(pixel_restore +1);
	else pixel = pixel_restore - 1;
	//this converts the pixelnumber to logical pixel coords
	pixel_x_log = pixel % xmax;
	pixel_y_log = pixel / xmax;

	//and simulate a user click with inverse mouseclick
	pixelHit(pixel_x_log, pixel_y_log, (pixel_restore < 0) ? Qt::LeftButton : Qt::RightButton);
	canvas->update();
}


/*
	this function is called when a pixel is hit, call it with the logical
	coords of a pixel, and with the button wich has hit the pixel
*/
void KpictorialView::pixelHit(int x, int y, Qt::ButtonState butt)	{

	int pixel_x, pixel_y;	//the real coords of the pixel (on the canvas)

	//and this to real coords like on the canvas
	pixel_x = x * pixsize + x_off + pixsize / 2;
	pixel_y = y * pixsize + y_off + pixsize / 2;

	/*
		I need this list, because it = (canvas->collisions( QPoint ( pixel_x, pixel_y))).begin();
		sometimes crashes at the line	(*it)->rtti() although the list is never empty, STRANGE
	*/
	QCanvasItemList list = canvas->collisions( QPoint ( pixel_x, pixel_y));
	
	if (list.isEmpty())	{
		qDebug("KpictorialView::pixelHit did not hit anything (%dx%d -> %dx%d)", x, y, pixel_x, pixel_y);
		return;
	}

	QCanvasItemList::Iterator it = list.begin();

	if ((*it)->rtti() != QCanvasRectangle::RTTI)	{
		qDebug("KpictorialView::pixelHit did not hit rectangle (%dx%d -> %dx%d)", x, y, pixel_x, pixel_y);
		return;
	}
	//ok, we now have the rectangle as *it, and the logical pixel coords, tkae the last click back


	if (butt == Qt::LeftButton)	*((char*)picture + x + y*xmax) += 1;
	if (butt == Qt::RightButton)	*((char*)picture + x + y*xmax) += 2;	//equals -=1 togethet with %=3
	 *((char*)picture + x + y*xmax)  %= 3;
	

	switch (*((char*)picture + x + y*xmax))	{
		case 0:((QCanvasRectangle*)(*it))->setBrush(*bwhite); break;
		case 1:((QCanvasRectangle*)(*it))->setBrush(*bblack); break;
		case 2:((QCanvasRectangle*)(*it))->setBrush(*bgray); break;
	}
}

/*
	calls this function whenever the user clicks on an pixel, and you also want the undo structure
	to be updates, warning, do not called with MouseButtons which are not assigned to any function!
*/
void KpictorialView::pixelHitUndo(int x, int y, Qt::ButtonState butt)	{
	pixelHit(x,y,butt);
	undo_push(x,y,butt);
}


/*
  	this invalidates the undo structure
*/
void KpictorialView::undo_flush()	{
	undo_first = undo_last = 0;
	emit signalUndoEmpty(true);
}

/*
	just call saving in doc
*/
bool KpictorialView::saveAs(QString filename)	{
	return doc->saveAs(filename);
}
