/***************************************************************************
                          kpictorialview.h  -  description
                             -------------------
    begin                : Thu Apr 18 2002
    copyright            : (C) 2002 by Roland Lezuo
    email                : roland.lezuo@chello.at
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KPICTORIALVIEW_H
#define KPICTORIALVIEW_H

#include <qcanvas.h>
#include <qcolor.h>
#include "kpictorialdoc.h"

/**
  *@author Roland Lezuo
  */

#define UNDO_SIZE	100

class KpictorialView : public QCanvasView  {

Q_OBJECT

	public:
	KpictorialView(QWidget *, int, int);				//creates random picture of given size (XxY)
	KpictorialView(QWidget *, KpictorialDoc *init_doc);	//creates picture from xpm/xbm/kdf file
	~KpictorialView();
	/*		returns: 	0, not the correct picture
					1, the correct one
					2, there are still gray field*/
	int checkPicture();
	//solves the picture for the user and markes correct and incorrect guesses, retuns percentage of correctness
	float solvePicture();		
	QSize size();			//retuns size of picture and text surroundning it
	int getPictureX();		//returns x size of picture
	int getPictureY();		//returns y size of picture
	const void *getPicture();			//retuns the picture
	const void *getUserPicture();		//returns the picture the user has guessed so far
	void undo_pop();				//undo the last click
	bool saveAs(QString);		//this just calls saving in the doc


	protected:
	void mousePressEvent(QMouseEvent *);		//takes clicks from user
	void createPicture();					//fills the canvas
	void init();							//initializes everything expect the document
	void undo_push(int, int, Qt::ButtonState);	//logical coords of pixel clicked + mouse button
	void undo_flush();						//this invalidates the undo structure
	void pixelHit(int, int, Qt::ButtonState);		//this function is called when a pixel hit occured
	void pixelHitUndo(int, int, Qt::ButtonState);		//this function is called when a pixel is clicked by user + updates undo

	private:
	QCanvas *canvas;
	KpictorialDoc *doc;			//instance of document containg the picture
	QBrush *bwhite, *bblack, *bgray;	//the brushes for the pixels
	QColor *gray, *red, *green;		//color of frame and lines seperating every 5 rows/columns
	QCanvasLine *line;				//for drawing the lines
	int pixsize, textsize;			//size of a pixel and size of letters
	int x_off, y_off, xmax, ymax;		//*_off...where the picture is beginning (offset from text) *max...size of picture
	void *picture;					//an array containting the state of pixels as clicked by user
	bool solved;					//when picture is solved, we don't take clicks any more
	int undo[UNDO_SIZE];			//this array saves clicks, so that undos can be done
	int undo_first, undo_last;		//as a zyclic array

	public slots:
	void slotResized();

	signals:
	void signalUndoEmpty(bool);			//this signals wheter there are undo data are not

};

#endif
